Class: Nucleon::Plugin::Node

Inherits:
Object
  • Object
show all
Includes:
Celluloid
Defined in:
lib/core/plugin/node.rb

Class Method Summary collapse

Instance Method Summary collapse

Dynamic Method Handling

This class handles dynamic methods through the method_missing method

#method_missing(method, *args, &code) ⇒ Object




50
51
52
53
54
# File 'lib/core/plugin/node.rb', line 50

def method_missing(method, *args, &code)  
  action(method, *args) do |op, data|
    code.call(op, data) if code
  end
end

Class Method Details

.build_info(type, data) ⇒ Object


Utilities



1019
1020
1021
1022
# File 'lib/core/plugin/node.rb', line 1019

def self.build_info(type, data)  
  data = data.split(/\s*,\s*/) if data.is_a?(String)
  super(type, data)
end

.translate(data) ⇒ Object




1026
1027
1028
1029
1030
1031
1032
1033
1034
1035
1036
1037
1038
1039
1040
1041
1042
1043
1044
1045
# File 'lib/core/plugin/node.rb', line 1026

def self.translate(data)
  options = super(data)
  
  case data        
  when String
    options = { :name => data }
  when Hash
    options = data
  end
  
  if options.has_key?(:name)
    if matches = translate_reference(options[:name])
      options[:provider] = matches[:provider]
      options[:name]     = matches[:name]
      
      logger.debug("Translating node options: #{options.inspect}")  
    end
  end
  options
end

.translate_reference(reference) ⇒ Object




1049
1050
1051
1052
1053
1054
1055
1056
1057
1058
1059
1060
1061
1062
1063
1064
1065
1066
# File 'lib/core/plugin/node.rb', line 1049

def self.translate_reference(reference)
  # ex: rackspace:::web1.staging.example.com
  if reference && reference.match(/^\s*([a-zA-Z0-9_-]+):::([^\s]+)\s*$/)
    provider = $1
    name     = $2
    
    logger.debug("Translating node reference: #{provider}  #{name}")
    
    info = {
      :provider => provider,
      :name     => name
    }
    
    logger.debug("Project reference info: #{info.inspect}")
    return info
  end
  nil
end

Instance Method Details

#action(provider, options = {}) ⇒ Object



655
656
657
658
659
660
661
662
663
664
665
666
667
668
669
670
671
672
673
674
675
676
677
678
679
680
681
# File 'lib/core/plugin/node.rb', line 655

def action(provider, options = {})
  codes :network_load_error
  
  config = Config.ensure(options).defaults({
    :log_level    => Nucleon.log_level,
    :net_remote   => :edit, 
    :net_provider => network.plugin_provider 
  })
  
  logger.info("Executing remote action #{provider} with encoded arguments: #{config.export.inspect}")
  
  encoded_config = Util::CLI.encode(Util::Data.clean(config.export))
  action_config  = extended_config(:action, {
    :command => provider, 
    :data    => { :encoded => encoded_config }
  })
  
  result = command(:corl, { :subcommand => action_config, :as_admin => true }) do |op, data|
    yield(op, data) if block_given?  
  end
  
  # Update local network configuration so we capture any updates
  if result.status == code.success && ! network.load({ :remote => config[:net_remote], :pull => true })
    result.status = code.network_load_error
  end
  result
end

#attach_keys(keypair) ⇒ Object




362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
391
# File 'lib/core/plugin/node.rb', line 362

def attach_keys(keypair)
  base_name   = "#{plugin_provider}-#{plugin_name}"
  save_config = { 
    :pull    => false, 
    :push    => false,
    :message => "Updating SSH keys for node #{plugin_provider} (#{plugin_name})" 
  }
  
  active  = machine && machine.running?
  result  = run.authorize({ :public_key => keypair.ssh_key }) if active
  success = false
  
  if ! active || result.status == code.success        
    private_key = network.attach_data(:keys, "#{base_name}-id_#{keypair.type}", keypair.encrypted_key)
    public_key  = network.attach_data(:keys, "#{base_name}-id_#{keypair.type}.pub", keypair.ssh_key)
  
    if private_key && public_key
      FileUtils.chmod(0600, private_key)
      FileUtils.chmod(0644, public_key)
    
      save_config[:files] = [ private_key, public_key ]
  
      myself.private_key = private_key
      myself.public_key  = public_key
  
      success = save(extended_config(:key_save, save_config))
    end
  end
  success
end

#bootstrap(local_path, options = {}) ⇒ Object




725
726
727
728
729
730
731
732
733
734
735
736
737
738
739
740
741
742
743
744
745
746
747
748
749
750
751
752
753
754
755
756
757
758
759
760
761
762
763
764
765
766
767
768
769
770
771
772
773
774
775
776
777
778
779
780
781
782
783
784
785
786
787
788
789
790
791
792
793
794
795
796
797
798
799
800
801
802
803
804
# File 'lib/core/plugin/node.rb', line 725

def bootstrap(local_path, options = {})
  config      = Config.ensure(options)
  myself.status = code.unknown_status
  
  bootstrap_name = 'bootstrap'    
  bootstrap_path = config.get(:bootstrap_path, File.join(CORL.lib_path, '..', bootstrap_name))
  bootstrap_glob = config.get(:bootstrap_glob, '**/*.sh')
  bootstrap_init = config.get(:bootstrap_init, 'bootstrap.sh')
  
  user_home  = config[:home]
  auth_files = config.get_array(:auth_files)
  
  codes :local_path_not_found,
        :home_path_lookup_failure,
        :auth_upload_failure,
        :bootstrap_upload_failure,
        :bootstrap_exec_failure,
        :reload_failure
  
  if File.directory?(local_path)      
    if user_home || user_home = home(config.get(:home_env_var, 'HOME'), config.get(:force, false))
      myself.status = code.success
      
      # Transmit authorisation / credential files
      package_files = [ '.fog', '.netrc', '.google-privatekey.p12', '.vimrc' ]
      auth_files.each do |file|
        package_files = file.gsub(local_path + '/', '')
      end
      send_success = send_files(local_path, user_home, package_files, '0600') do |op, data|
        yield("send_#{op}".to_sym, data) if block_given?
        data
      end
      unless send_success
        myself.status = code.auth_upload_failure
      end
  
      # Send bootstrap package
      if status == code.success
        remote_bootstrap_path = File.join(user_home, bootstrap_name)
        
        cli.rm('-Rf', remote_bootstrap_path)
        send_success = send_files(bootstrap_path, remote_bootstrap_path, nil, '0700') do |op, data|
          yield("send_#{op}".to_sym, data) if block_given?
          data
        end
        unless send_success
          myself.status = code.bootstrap_upload_failure
        end
        
        # Execute bootstrap process
        if status == code.success
          remote_script = File.join(remote_bootstrap_path, bootstrap_init)
          
          myself[:bootstrap] = remote_script
          
          result = command("HOSTNAME='#{hostname}' #{remote_script}", { :as_admin => true }) do |op, data|
            yield("exec_#{op}".to_sym, data) if block_given?
            data
          end
          
          if result.status == code.success
            # Reboot the machine
            unless reload
              warn('corl.core.node.bootstrap.reload')
              myself.status = code.reload_failure 
            end
          else
            warn('corl.core.node.bootstrap.status', { :script => remote_script, :status => result.status })
            myself.status = code.bootstrap_exec_failure
          end
        end
      end
    else
      myself.status = code.home_path_lookup_failure            
    end
  else
    myself.status = code.local_path_not_found
  end
  status == code.success
end

#build(options = {}) ⇒ Object


Machine operations



330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
# File 'lib/core/plugin/node.rb', line 330

def build(options = {})
  config  = Config.ensure(options)
  success = true
  
  provisioners.each do |provider, collection|
    ui.info("Building #{provider} provisioner collection")
        
    collection.each do |name, provisioner|
      ui.info("Building #{name} provisioner")
      
      unless provisioner.build(options)
        success = false
        break
      end
    end
  end
  
  if success
    ui.success("Saving successful build")
    
    myself[:build] = Time.now.to_s
  
    success = save(extended_config(:build, {
      :message => config.get(:message, "Built #{plugin_provider} node: #{plugin_name}"),
      :remote  => config.get(:remote, :edit)  
    }))
  end
  success
end

#build_timeObject




311
312
313
# File 'lib/core/plugin/node.rb', line 311

def build_time
  myself[:build]
end

#cliObject




623
624
625
# File 'lib/core/plugin/node.rb', line 623

def cli
  @cli_interface
end

#cli_capture(cli_command, *args) ⇒ Object


CLI utilities



1077
1078
1079
1080
1081
1082
1083
1084
1085
# File 'lib/core/plugin/node.rb', line 1077

def cli_capture(cli_command, *args)
  result = cli.send(cli_command, args)
           
  if result.status == code.success && ! result.output.empty? 
    result.output
  else
    nil
  end  
end

#cli_check(cli_command, *args) ⇒ Object




1089
1090
1091
1092
# File 'lib/core/plugin/node.rb', line 1089

def cli_check(cli_command, *args)
  result = cli.send(cli_command, args)
  result.status == code.success ? true : false  
end

#command(command, options = {}) ⇒ Object



631
632
633
634
635
636
637
638
639
640
641
642
643
644
645
646
647
648
649
# File 'lib/core/plugin/node.rb', line 631

def command(command, options = {})
  config   = Config.ensure(options)
  as_admin = config.delete(:as_admin, false)
  
  unless command.is_a?(CORL::Plugin::Command)
    command = CORL.command(Config.new({ :command => command }).import(config), :bash)
  end
  
  admin_command = ''
  if as_admin
    admin_command = 'sudo' if user.to_s == 'ubuntu'
    admin_command = extension_set(:admin_command, admin_command, config)
  end
  
  results = exec({ :commands => [ "#{admin_command} #{command.to_s}".strip ] }) do |op, data|
    yield(op, data) if block_given?  
  end
  results.first
end

#create(options = {}) ⇒ Object




433
434
435
436
437
438
439
440
441
442
443
444
445
446
447
448
449
450
451
452
453
454
455
456
457
458
# File 'lib/core/plugin/node.rb', line 433

def create(options = {})
  success = true
  
  if machine
    config = Config.ensure(options)
    
    if extension_check(:create, { :config => config })
      logger.info("Creating node: #{plugin_name}")
    
      yield(:config, config) if block_given?      
      success = machine.create(config.export)
      
      if success && block_given?
        process_success = yield(:process, config)
        success         = process_success if process_success == false        
      end
      
      if success
        extension(:create_success, { :config => config })
      end
    end
  else
    logger.warn("Node #{plugin_name} does not have an attached machine so cannot be created")
  end
  success
end

#create_image(options = {}) ⇒ Object




902
903
904
905
906
907
908
909
910
911
912
913
914
915
916
917
918
919
920
921
922
923
924
925
926
927
928
# File 'lib/core/plugin/node.rb', line 902

def create_image(options = {})
  success = true
  
  if machine && machine.running?
    config = Config.ensure(options)
    
    if extension_check(:create_image, { :config => config })
      logger.info("Executing node: #{plugin_name}")
    
      yield(:config, config) if block_given?      
      success = machine.create_image(config.export)
      success = save(config) if success
      
      if success && block_given?
        process_success = yield(:process, config)
        success         = process_success if process_success == false        
      end
      
      if success
        extension(:create_image_success, { :config => config })
      end
    end
  else
    logger.warn("Node #{plugin_name} does not have an attached machine or is not running so cannot create an image")
  end
  success   
end

#create_machine(name, provider, options = {}) ⇒ Object




427
428
429
# File 'lib/core/plugin/node.rb', line 427

def create_machine(name, provider, options = {})
  CORL.create_plugin(:machine, provider, extended_config(name, options).import({ :meta => { :parent => myself }}))
end

#delete_keysObject




395
396
397
398
399
400
401
402
403
404
405
406
407
408
409
410
411
412
413
414
415
416
417
418
419
420
421
422
423
# File 'lib/core/plugin/node.rb', line 395

def delete_keys
  private_key = myself[:private_key]
  public_key  = myself[:public_key]
  
  keys = []
  keys << private_key if private_key
  keys << public_key if public_key
  
  success = true
  
  unless keys.empty?
    files = network.delete_attachments(keys)
    
    if files && ! files.empty?
      delete_setting(:private_key)
      delete_setting(:public_key)
      
      success = save(extended_config(:key_delete, {
        :files   => [ private_key, public_key ], 
        :pull    => false, 
        :push    => false,
        :message => "Removing SSH keys for node #{plugin_provider} (#{plugin_name})"  
      }))
    else
      success = false
    end
  end
  success
end

#destroy(options = {}) ⇒ Object




973
974
975
976
977
978
979
980
981
982
983
984
985
986
987
988
989
990
991
992
993
994
995
996
997
998
999
1000
1001
1002
1003
1004
1005
1006
1007
1008
1009
1010
1011
1012
1013
1014
# File 'lib/core/plugin/node.rb', line 973

def destroy(options = {})    
  success = true
  
  if machine && machine.created?
    config = Config.ensure(options)
    
    if extension_check(:destroy, { :config => config })
      logger.info("Destroying node: #{plugin_name}")
    
      yield(:config, config) if block_given?
      
      # Shut down machine      
      success = machine.destroy(config.export)
      
      myself.machine = nil
      
      # Remove SSH keys
      if success && delete_keys
        # Remove node information
        network.delete_node(plugin_provider, plugin_name, false)
        
        network.save({
          :commit => true, 
          :remote => config.get(:remote, :edit), 
          :push   => true 
        })
      end      
      
      if success && block_given?
        process_success = yield(:process, config)
        success         = process_success if process_success == false        
      end
      
      if success
        extension(:destroy_success, { :config => config })
      end
    end
  else
    logger.warn("Node #{plugin_name} does not have an attached machine or is not created so cannot be destroyed")
  end
  success    
end

#download(remote_path, local_path, options = {}) ⇒ Object




462
463
464
465
466
467
468
469
470
471
472
473
474
475
476
477
478
479
480
481
482
483
484
485
486
487
488
489
490
491
492
493
494
495
496
# File 'lib/core/plugin/node.rb', line 462

def download(remote_path, local_path, options = {})
  success = false
  
  if machine && machine.running?
    config      = Config.ensure(options)
    hook_config = Config.new({ :local_path => local_path, :remote_path => remote_path, :config => config })
    
    if extension_check(:download, hook_config)
      logger.info("Downloading from #{plugin_name}")
    
      render("Starting download of #{remote_path} to #{local_path}") 
      yield(:config, hook_config) if block_given?
      
      active_machine = local? ? local_machine : machine
      
      success = active_machine.download(remote_path, local_path, config.export) do |name, received, total|
        render("#{name}: Sent #{received} of #{total}")
        yield(:progress, { :name => name, :received => received, :total => total })
      end
      
      if success && block_given?
        render("Successfully finished download of #{remote_path} to #{local_path}")
        process_success = yield(:process, hook_config)
        success         = process_success if process_success == false        
      end
      
      if success
        extension(:download_success, hook_config)
      end
    end
  else
    logger.warn("Node #{plugin_name} does not have an attached machine or is not running so cannot download")
  end
  success
end

#exec(options = {}) ⇒ Object



573
574
575
576
577
578
579
580
581
582
583
584
585
586
587
588
589
590
591
592
593
594
595
596
597
598
599
600
601
602
603
604
605
606
607
608
609
610
611
612
613
614
615
616
617
618
619
# File 'lib/core/plugin/node.rb', line 573

def exec(options = {})
  results = nil
  
  if machine && machine.running?
    config = Config.ensure(options)
    
    if extension_check(:exec, { :config => config })
      logger.info("Executing node: #{plugin_name}")
            
      yield(:config, config) if block_given?
      
      if local? && local_machine
        active_machine = local_machine
        
        config[:info_prefix]  = "[#{hostname}] "
        config[:error_prefix] = "[#{hostname}] "
      else
        active_machine = machine
      end
      
      if commands = config.get(:commands, nil)
        results = active_machine.exec(commands, config.export) do |type, command, data|
          unless local?
            if type == :error
              alert(data.gsub(/^(\e\[\d\d?m)?\[[^\]]+\]\s*/, '\\1'))
            else
              render(data.gsub(/^(\e\[\d\d?m)?\[[^\]]+\]\s*/, '\\1'))
            end
          end
          yield(:progress, { :type => type, :command => command, :data => data }) if block_given?   
        end
      end
      
      success  = true
      results.each do |result|
        success = false if result.status != code.success  
      end
      if success
        yield(:process, config) if block_given?
        extension(:exec_success, { :config => config, :results => results }) 
      end
    end
  else
    logger.warn("Node #{plugin_name} does not have an attached machine or is not running so cannot execute commands")
  end
  results 
end

#home(env_var = 'HOME', reset = false) ⇒ Object




163
164
165
166
167
168
# File 'lib/core/plugin/node.rb', line 163

def home(env_var = 'HOME', reset = false)
  if reset || myself[:user_home].nil?
    myself[:user_home] = cli_capture(:echo, '$' + env_var.to_s.gsub('$', '')) if machine
  end
  myself[:user_home]
end

#hostnameObject




122
123
124
125
126
127
128
129
130
# File 'lib/core/plugin/node.rb', line 122

def hostname
  hostname = myself[:hostname]
  
  if hostname.to_s != ui.resource.to_s 
    ui.resource = Util::Console.colorize(hostname, @class_color)
    logger      = hostname
  end
  hostname
end

#id(reset = false) ⇒ Object




103
104
105
106
# File 'lib/core/plugin/node.rb', line 103

def id(reset = false)
  myself[:id] = machine.plugin_name if machine && ( reset || myself[:id].nil? )
  myself[:id]
end

#image(reset = false) ⇒ Object



267
268
269
270
# File 'lib/core/plugin/node.rb', line 267

def image(reset = false)
  myself[:image] = machine.image if machine && ( reset || myself[:image].nil? )
  myself[:image]
end

#image=(image) ⇒ Object




263
264
265
# File 'lib/core/plugin/node.rb', line 263

def image=image
  myself[:image] = image
end

#image_id(image) ⇒ Object


Image utilities



1110
1111
1112
# File 'lib/core/plugin/node.rb', line 1110

def image_id(image)
  image.id
end

#image_search_text(image) ⇒ Object




1122
1123
1124
# File 'lib/core/plugin/node.rb', line 1122

def image_search_text(image)
  image.to_s
end

#images(search_terms = [], options = {}) ⇒ Object




224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
# File 'lib/core/plugin/node.rb', line 224

def images(search_terms = [], options = {})
  config = Config.ensure(options)
  images = []
  
  if machine
    loaded_images = machine.images
  
    if loaded_images
      require_all = config.get(:require_all, false)
      
      loaded_images.each do |image|
        if usable_image?(image)
          include_image = ( search_terms.empty? ? true : require_all )
          image_text    = image_search_text(image)
          
          search_terms.each do |term|
            if config.get(:match_case, false)
              success = image_text.match(/#{term}/)
            else
              success = image_text.match(/#{term}/i)  
            end
            
            if require_all            
              include_image = false unless success
            else
              include_image = true if success  
            end
          end
      
          images << image if include_image
        end
      end
    end
  end
  images
end

#local?Boolean


Checks

Returns:

  • (Boolean)


66
67
68
# File 'lib/core/plugin/node.rb', line 66

def local?
  @local_context ? true : false
end

#local_machineObject




93
94
95
# File 'lib/core/plugin/node.rb', line 93

def local_machine
  @local_machine
end

#local_machine=(local_machine) ⇒ Object



97
98
99
# File 'lib/core/plugin/node.rb', line 97

def local_machine=local_machine
  @local_machine = local_machine  
end

#localizeObject




58
59
60
61
# File 'lib/core/plugin/node.rb', line 58

def localize
  @local_context       = true
  myself.local_machine = create_machine(:local_machine, :physical)
end

#machineObject




83
84
85
# File 'lib/core/plugin/node.rb', line 83

def machine
  @machine
end

#machine=(machine) ⇒ Object



87
88
89
# File 'lib/core/plugin/node.rb', line 87

def machine=machine
  @machine = machine  
end

#machine_config {|config| ... } ⇒ Object


Settings groups

Yields:

  • (config)


318
319
320
321
322
323
324
325
# File 'lib/core/plugin/node.rb', line 318

def machine_config
  name   = myself[:id]
  name   = myself[:hostname] if name.nil?
  config = Config.new({ :name => name })
  
  yield(config) if block_given?
  config
end

#machine_type(reset = false) ⇒ Object



206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
# File 'lib/core/plugin/node.rb', line 206

def machine_type(reset = false)
  myself[:machine_type] = machine.machine_type if machine && ( reset || myself[:machine_type].nil? )
  machine_type          = myself[:machine_type]
  
  if machine_type.nil? && machine
    if types = machine_types
      unless types.empty?
        machine_type          = machine_type_id(types.first)
        myself[:machine_type] = machine_type
      end
    end
  end
  
  machine_type
end

#machine_type_id(machine_type) ⇒ Object


Machine type utilities



1097
1098
1099
# File 'lib/core/plugin/node.rb', line 1097

def machine_type_id(machine_type)
  machine_type.id
end

#machine_typesObject




202
203
204
# File 'lib/core/plugin/node.rb', line 202

def machine_types # Must be set at machine level (queried)
  machine.machine_types if machine
end

#normalize(reload) ⇒ Object


Node plugin interface



12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
# File 'lib/core/plugin/node.rb', line 12

def normalize(reload)
  super
  
  @class_color = :purple
  
  export.each do |name, value|
    myself[name] = value
  end
  
  yield if block_given? # Chance to create a machine to feed hostname
  
  ui.resource = Util::Console.colorize(hostname, @class_color)
  logger      = hostname
  
  myself[:settings] = [ "all", plugin_provider.to_s, plugin_name.to_s ] | setting(:settings, [], :array)
  
  unless reload
    @cli_interface = Util::Liquid.new do |method, args, &code|
      result = exec({ :commands => [ [ method, args ].flatten.join(' ') ] }) do |op, data|
        code.call(op, data) if code
      end
      if result
        result = result.first
        alert(result.errors) unless result.errors.empty?
      end
      result
    end
  
    @action_interface = Util::Liquid.new do |method, args, &code|
      action(method, *args) do |op, data|
        code.call(op, data) if code
      end
    end
  end
end

#private_ip(reset = false) ⇒ Object



115
116
117
118
# File 'lib/core/plugin/node.rb', line 115

def private_ip(reset = false)
  myself[:private_ip] = machine.private_ip if machine && ( reset || myself[:private_ip].nil? )
  myself[:private_ip]
end

#private_keyObject



183
184
185
186
187
# File 'lib/core/plugin/node.rb', line 183

def private_key
  config_key = myself[:private_key]
  return File.expand_path(config_key, network.directory) if config_key
  nil
end

#private_key=(private_key) ⇒ Object




179
180
181
# File 'lib/core/plugin/node.rb', line 179

def private_key=private_key
  myself[:private_key] = private_key
end

#profilesObject



278
279
280
# File 'lib/core/plugin/node.rb', line 278

def profiles
  array(myself[:profiles])
end

#profiles=(profiles) ⇒ Object




274
275
276
# File 'lib/core/plugin/node.rb', line 274

def profiles=profiles
  myself[:profiles] = array(profiles)
end

#provisioner_infoObject




284
285
286
287
288
289
290
291
292
293
294
295
296
297
# File 'lib/core/plugin/node.rb', line 284

def provisioner_info
  provisioner_info = {}
      
  # Compose needed provisioners and profiles
  profiles.each do |profile|        
    if info = Plugin::Provisioner.translate_reference(profile)
      provider = info[:provider]
          
      provisioner_info[provider] = { :profiles => [] } unless provisioner_info.has_key?(provider)
      provisioner_info[provider][:profiles] += info[:profiles]
    end
  end
  provisioner_info
end

#provisionersObject




301
302
303
304
305
306
307
# File 'lib/core/plugin/node.rb', line 301

def provisioners
  provisioners = {}        
  provisioner_info.each do |provider, node_profiles|
    provisioners[provider] = network.provisioners(provider)
  end    
  provisioners
end

#public_ip(reset = false) ⇒ Object




110
111
112
113
# File 'lib/core/plugin/node.rb', line 110

def public_ip(reset = false)
  myself[:public_ip] = machine.public_ip if machine && ( reset || myself[:public_ip].nil? )
  myself[:public_ip]
end

#public_keyObject



195
196
197
198
# File 'lib/core/plugin/node.rb', line 195

def public_key
  config_key = myself[:public_key]
  return File.expand_path(config_key, network.directory) if config_key
end

#public_key=(public_key) ⇒ Object




191
192
193
# File 'lib/core/plugin/node.rb', line 191

def public_key=public_key
  myself[:public_key] = public_key
end

#reload(options = {}) ⇒ Object




873
874
875
876
877
878
879
880
881
882
883
884
885
886
887
888
889
890
891
892
893
894
895
896
897
898
# File 'lib/core/plugin/node.rb', line 873

def reload(options = {})
  success = true
  
  if machine && machine.created?
    config = Config.ensure(options)
    
    if extension_check(:reload, { :config => config })
      logger.info("Reloading node: #{plugin_name}")
    
      yield(:config, config) if block_given?      
      success = machine.reload(config.export)
      
      if success && block_given?
        process_success = yield(:process, config)
        success         = process_success if process_success == false        
      end
      
      if success
        extension(:reload_success, { :config => config })
      end
    end
  else
    logger.warn("Node #{plugin_name} does not have an attached machine or is not created so cannot be reloaded")
  end
  success
end

#render_image(image) ⇒ Object




1116
1117
1118
# File 'lib/core/plugin/node.rb', line 1116

def render_image(image)
  ''  
end

#render_machine_type(machine_type) ⇒ Object




1103
1104
1105
# File 'lib/core/plugin/node.rb', line 1103

def render_machine_type(machine_type)
  ''  
end

#runObject




685
686
687
# File 'lib/core/plugin/node.rb', line 685

def run
  @action_interface
end

#save(options = {}) {|config| ... } ⇒ Object


Yields:

  • (config)


808
809
810
811
812
813
814
815
816
817
818
819
820
821
822
823
824
825
826
827
828
829
# File 'lib/core/plugin/node.rb', line 808

def save(options = {})
  config = Config.ensure(options)
  
  # Record machine parameters
  id(true)
  public_ip(true)
  private_ip(true)
  state(true)
  
  machine_type(false)
  image(false)
  
  # Provider or external configuration preparation
  yield(config) if block_given?
  
  network.save(config.import({ 
    :commit      => true,
    :allow_empty => true,
    :message     => config.get(:message, "Saving #{plugin_provider} node #{plugin_name}"),
    :remote      => config.get(:remote, :edit)
  }))    
end

#send_files(local_path, remote_path, files = nil, permission = '0644', &code) ⇒ Object




538
539
540
541
542
543
544
545
546
547
548
549
550
551
552
553
554
555
556
557
558
559
560
561
562
563
564
565
566
567
# File 'lib/core/plugin/node.rb', line 538

def send_files(local_path, remote_path, files = nil, permission = '0644', &code)
  local_path = File.expand_path(local_path)
  return false unless File.directory?(local_path)
  
  success = true
  
  send_file = lambda do |local_file, remote_file|
    send_success = upload(local_file, remote_file) do |op, options|
      code.call(op, options) if code
    end
    send_success = cli_check(:chmod, permission, remote_file) if send_success
    send_success  
  end
  
  if files && files.is_a?(Array)
    files.flatten.each do |rel_file_name|
      local_file  = "#{local_path}/#{rel_file_name}"
      remote_file = "#{remote_path}/#{rel_file_name}"
  
      if File.exists?(local_file)
        send_success = send_file.call(local_file, remote_file) 
        success      = false unless send_success
      end
    end
  else
    send_success = send_file.call(local_path, remote_path)
    success      = false unless send_success
  end
  success
end

#ssh_path(home_env_var = 'HOME', reset = false) ⇒ Object




172
173
174
175
# File 'lib/core/plugin/node.rb', line 172

def ssh_path(home_env_var = 'HOME', reset = false)
  home = home(home_env_var, reset)
  home ? File.join(home, '.ssh') : nil
end

#ssh_portObject



156
157
158
159
# File 'lib/core/plugin/node.rb', line 156

def ssh_port
  myself[:ssh_port] = 22 if myself[:ssh_port].nil?
  myself[:ssh_port]
end

#ssh_port=(ssh_port) ⇒ Object




151
152
153
154
# File 'lib/core/plugin/node.rb', line 151

def ssh_port=ssh_port
  myself[:ssh_port] = ssh_port
  machine.init_ssh(ssh_port) if machine
end

#start(options = {}) ⇒ Object




833
834
835
836
837
838
839
840
841
842
843
844
845
846
847
848
849
850
851
852
853
854
855
856
857
858
859
860
861
862
863
864
865
866
867
868
869
# File 'lib/core/plugin/node.rb', line 833

def start(options = {})
  success = true
  
  if machine
    config = Config.ensure(options)
    
    if extension_check(:start, { :config => config })
      logger.info("Starting node: #{plugin_name}")
    
      yield(:config, config) if block_given?      
      success = machine.start(config.export)
      success = save(config) if success
      
      if success
        if bootstrap_script = myself[:bootstrap]
          result = command("HOSTNAME='#{hostname}' #{bootstrap_script}", { :as_admin => true }) do |op, data|
            yield("bootstrap_#{op}".to_sym, data) if block_given?
            data
          end
          success = false unless result.status == code.success && reload
        end
      end
      
      if success && block_given?
        process_success = yield(:process, config)
        success         = process_success if process_success == false        
      end
      
      if success
        extension(:start_success, { :config => config })
      end
    end
  else
    logger.warn("Node #{plugin_name} does not have an attached machine so cannot be started")
  end
  success
end

#state(reset = false) ⇒ Object




134
135
136
137
# File 'lib/core/plugin/node.rb', line 134

def state(reset = false)
  myself[:state] = machine.state if machine && ( reset || myself[:state].nil? )
  myself[:state]
end

#stop(options = {}) ⇒ Object




932
933
934
935
936
937
938
939
940
941
942
943
944
945
946
947
948
949
950
951
952
953
954
955
956
957
958
959
960
961
962
963
964
965
966
967
968
969
# File 'lib/core/plugin/node.rb', line 932

def stop(options = {})
  success = true
  
  if machine && machine.running?
    config = Config.ensure(options)
    
    if extension_check(:stop, { :config => config })
      logger.info("Stopping node: #{plugin_name}")
    
      yield(:config, config) if block_given?      
      success = machine.stop(config.export)
      
      myself.machine = nil
      
      delete_setting(:id)
      delete_setting(:public_ip)
      delete_setting(:private_ip)
      delete_setting(:ssh_port)
      delete_setting(:build)
      
      myself[:state] = :stopped
      
      success = save(config) if success
      
      if success && block_given?
        process_success = yield(:process, config)
        success         = process_success if process_success == false        
      end
      
      if success
        extension(:stop_success, { :config => config })
      end
    end
  else
    logger.warn("Node #{plugin_name} does not have an attached machine or is not running so cannot be stopped")
  end
  success
end

#terminal(options = {}) ⇒ Object




691
692
693
694
695
696
697
698
699
700
701
702
703
704
705
706
707
708
709
710
711
712
713
714
715
716
717
718
719
720
721
# File 'lib/core/plugin/node.rb', line 691

def terminal(options = {})
  myself.status = code.unknown_status
  
  if machine && machine.running?
    config = Config.ensure(options)
    
    if extension_check(:terminal, { :config => config })
      logger.info("Launching terminal for node: #{plugin_name}")
      
      if local? && local_machine
        active_machine = local_machine
        
        config[:info_prefix]  = "[#{hostname}] "
        config[:error_prefix] = "[#{hostname}] "
      else
        active_machine = machine
      end
      
      config[:private_keys] = private_key
      
      myself.status = active_machine.terminal(user, config.export)
      
      if status == code.success
        extension(:exec_success, { :config => config }) 
      end
    end
  else
    logger.warn("Node #{plugin_name} does not have an attached machine or is not running so cannot launch terminal")
  end
  status == code.success
end

#translate_reference(reference) ⇒ Object




1070
1071
1072
# File 'lib/core/plugin/node.rb', line 1070

def translate_reference(reference)
  myself.class.translate_reference(reference)
end

#upload(local_path, remote_path, options = {}) ⇒ Object




500
501
502
503
504
505
506
507
508
509
510
511
512
513
514
515
516
517
518
519
520
521
522
523
524
525
526
527
528
529
530
531
532
533
534
# File 'lib/core/plugin/node.rb', line 500

def upload(local_path, remote_path, options = {})
  success = false
  
  if machine && machine.running?
    config      = Config.ensure(options)
    hook_config = Config.new({ :local_path => local_path, :remote_path => remote_path, :config => config })
    
    if extension_check(:upload, hook_config)
      logger.info("Uploading to #{plugin_name}")
    
      render("Starting upload of #{local_path} to #{remote_path}") 
      yield(:config, hook_config) if block_given?
      
      active_machine = local? ? local_machine : machine
      
      success = active_machine.upload(local_path, remote_path, config.export) do |name, sent, total|
        render("#{name}: Sent #{sent} of #{total}")
        yield(:progress, { :name => name, :sent => sent, :total => total })  
      end
      
      if success && block_given?
        render("Successfully finished upload of #{local_path} to #{remote_path}")
        process_success = yield(:process, hook_config)
        success         = process_success if process_success == false        
      end
      
      if success
        extension(:upload_success, hook_config)
      end
    end
  else
    logger.warn("Node #{plugin_name} does not have an attached machine or is not running so cannot upload")
  end
  success
end

#usable_image?(image) ⇒ Boolean


Returns:

  • (Boolean)


72
73
74
# File 'lib/core/plugin/node.rb', line 72

def usable_image?(image)
  true
end

#userObject



145
146
147
# File 'lib/core/plugin/node.rb', line 145

def user
  myself[:user]
end

#user=(user) ⇒ Object




141
142
143
# File 'lib/core/plugin/node.rb', line 141

def user=user
  myself[:user] = user
end